import { dcaIcon } from './icons';
import { ToolbarButton } from '@jupyterlab/apputils';
import { initMixpanel } from './mixpanel';
import { INotebookTracker, NotebookActions } from "@jupyterlab/notebook";
import { toArray, toObject } from '@lumino/algorithm';
import '../style/style.css';
/**
 * The command IDs used by the plugin.
 */
export var CommandIDs;
(function (CommandIDs) {
    CommandIDs.dcaInit = 'notebook:init-domino-code-assist';
})(CommandIDs || (CommandIDs = {}));
class DCAInitButton {
    constructor(commands, notebookTracker) {
        this._commands = commands;
        this._notebookTracker = notebookTracker;
    }
    createNew(panel) {
        const nb = this._notebookTracker;
        const button = new ToolbarButton({
            className: 'dcaInit',
            tooltip: 'Initialize Domino Code Assist',
            label: "Domino Code Assist",
            icon: dcaIcon,
            onClick: () => {
                const current = panel;
                NotebookActions.insertAbove(current.content);
                const activeCell = current.content.activeCell;
                if (activeCell) {
                    activeCell.model.value.text = "# this code is generated by the Domino Code Assist toolbar button\nimport domino_code_assist as dca\ndca.init()";
                    NotebookActions.runAndAdvance(current.content, current.sessionContext);
                }
            }
        });
        panel.toolbar.insertAfter('cellType', 'dcaInit', button);
        return button;
    }
}
const plugin = {
    id: 'dcalabextension:plugin',
    autoStart: true,
    requires: [INotebookTracker],
    activate: (app, NotebookTracker) => {
        initMixpanel();
        const getNotebookId = (notebookChildElement) => {
            const target = NotebookTracker.find(notebook => notebook.node.contains(notebookChildElement));
            return target === null || target === void 0 ? void 0 : target.id;
        };
        const getCells = (notebookId) => {
            const notebook = NotebookTracker.find(notebook => notebook.id === notebookId);
            if (notebook) {
                return notebook.content.widgets.map(cell => cell);
            }
        };
        const changeCellType = (notebookId, cell, cellType) => {
            var _a;
            const notebook = NotebookTracker.find(notebook => notebook.id === notebookId);
            const cellIndex = notebook === null || notebook === void 0 ? void 0 : notebook.content.widgets.indexOf(cell);
            if (notebook && notebook.model && cellIndex !== undefined) {
                const cellData = cell.model.toJSON();
                let newCell = null;
                if (cellType === 'code') {
                    newCell = notebook.model.contentFactory.createCodeCell({ cell: cellData });
                }
                else if (cellType === 'markdown') {
                    newCell = notebook.model.contentFactory.createMarkdownCell({ cell: cellData });
                }
                if (newCell !== null) {
                    (_a = notebook.model) === null || _a === void 0 ? void 0 : _a.cells.set(cellIndex, newCell);
                }
            }
        };
        const insertCellAbove = (notebookId, cell, value) => {
            const notebook = NotebookTracker.find(notebook => notebook.id === notebookId);
            if (notebook) {
                const cellIndex = notebook === null || notebook === void 0 ? void 0 : notebook.content.widgets.indexOf(cell);
                notebook.content.activeCellIndex = cellIndex;
                NotebookActions.insertAbove(notebook.content);
                const activeCell = notebook.content.activeCell;
                if (activeCell) {
                    activeCell.model.value.text = value.code;
                    if (value.meta) {
                        activeCell.model.metadata.set('assistant', value.meta);
                    }
                }
                NotebookActions.runAndAdvance(notebook.content, notebook.sessionContext);
            }
        };
        const insertCellsBelow = (notebookId, cell, cells) => {
            const notebook = NotebookTracker.find(notebook => notebook.id === notebookId);
            if (notebook) {
                const cellIndex = notebook === null || notebook === void 0 ? void 0 : notebook.content.widgets.indexOf(cell);
                notebook.content.activeCellIndex = cellIndex;
                const idMap = {};
                cells.forEach(({ code, meta, type_, id }) => {
                    NotebookActions.insertBelow(notebook.content);
                    let activeCell = notebook.content.activeCell;
                    if (activeCell && type_ === 'markdown') {
                        changeCellType(notebookId, activeCell, 'markdown');
                        activeCell = notebook.content.activeCell;
                    }
                    if (activeCell) {
                        if (id) {
                            idMap[id] = activeCell.model.id;
                        }
                        if (code.includes(Object.keys(idMap)[0])) {
                            code = Object.entries(idMap).reduce((result, [key, value]) => {
                                return result.replace(key, value);
                            }, code);
                            meta = meta && Object.entries(idMap).reduce((result, [key, value]) => {
                                return result.replace(key, value);
                            }, meta);
                        }
                        activeCell.model.value.text = code;
                        if (meta) {
                            activeCell.model.metadata.set('assistant', meta);
                        }
                    }
                    NotebookActions.run(notebook.content, notebook.sessionContext);
                });
            }
        };
        const runAndAdvance = (notebookId) => {
            const notebook = NotebookTracker.find(notebook => notebook.id === notebookId);
            if (notebook) {
                NotebookActions.runAndAdvance(notebook.content, notebook.sessionContext);
            }
        };
        const getNotebookPath = (notebookId) => {
            const notebook = NotebookTracker.find(notebook => notebook.id === notebookId);
            if (notebook) {
                return notebook.context.path;
            }
        };
        const saveNotebook = async (notebookId) => {
            const notebook = NotebookTracker.find(notebook => notebook.id === notebookId);
            if (notebook) {
                try {
                    await notebook.context.save();
                    return true;
                }
                catch (e) { }
            }
            return false;
        };
        const activate = (notebookId, cell) => {
            /* cell.activate() doesn't work for markdown cells */
            const notebook = NotebookTracker.find(notebook => notebook.id === notebookId);
            if (notebook) {
                const cellIndex = notebook.content.widgets.indexOf(cell);
                notebook.content.activeCellIndex = cellIndex;
            }
        };
        const assistants = {}; // maps from notebookId to assistant vue object
        const DCA = { frontend: 'lab', toArray, toObject, getNotebookId, getCells, assistants, changeCellType, insertCellAbove, insertCellsBelow, runAndAdvance, getNotebookPath, saveNotebook, activate };
        window["DCA"] = DCA;
        console.info('JupyterLab extension dcalabextension is activated!', NotebookTracker);
        const { commands, docRegistry } = app;
        if (NotebookTracker) {
            const dcaButton = new DCAInitButton(commands, NotebookTracker);
            docRegistry.addWidgetExtension('Notebook', dcaButton);
        }
    }
};
export default plugin;
